function results = run_M2C2Frgbt(seq, res_path, bSaveImage) 
%  M2C2Frgbt tracker enhances the robustness of the ECO baseline [1] for rgbt visual tracking.
%  [1]	M. Danelljan, G. Bhat, F. S. Khan, and M. Felsberg, ...
%  ECO: Efficient convolution operators for tracking, in CVPR, 2017, pp. 66386645.
%  The algorithm has been developed by: Alireza Memarmoghadam, April 2020
%  {email: a.memarmoghadam@eng.ui.ac.ir and a.memarmoghadam@yahoo.com} 
%

%- Platform check.
if nargin < 1
  switch computer('arch')
    case {'win32', 'win64', 'glnx86', 'glnx64', 'maci64'}
      results = {};  %- Supported platforms. Do nothing.
    case {}
      error(['Unsupported planform - ' computer('arch') '.']);
    otherwise
      error(['Unknown planform - ' computer('arch') '.']);
  end
  return;
end

if nargin < 4, run_opt = struct('dumppath_fmt','-', 'tracker_path','./'); end;

% Add paths
setup_paths();

% Initialize the tracker parameters
benchmark_features_type = 'VOT_HC'; %: 'OTB_HC', 'OTB_Deep', 'VOT_HC', 'VOT_Deep'
switch benchmark_features_type,
    case 'OTB_HC'
        params = default_parameters_ECO_OTB_HC();
    case 'OTB_Deep'
        params = default_parameters_ECO_OTB_Deep();
    case 'VOT_HC'
        params = default_parameters_ECO_VOT_HC();
    case 'VOT_Deep'
        params = default_parameters_ECO_VOT_Deep();
end

% Initialize
params.seq = seq;

% Run tracker
results = tracker(params);

end
